use strict;
use warnings;
use Test::More;

# generated by Dist::Zilla::Plugin::Test::PodSpelling 2.006008
use Test::Spelling 0.12;
use Pod::Wordlist;


add_stopwords(<DATA>);
all_pod_files_spelling_ok( qw( bin lib  ) );
__DATA__
Readonly
committer
committers
mailmap
AnnoCPAN
CPAN
io
IO
Github
HTTPS
JSON
POSIX
Params
Storable
Subdirectory
TODO
Timestamp
alnum
ascii
aws
cas
dir
filename
http
https
json
licensable
lv
msg
multi
params
runtime
scalability
semafores
startup
subitem
subparam
succcessful
successfull
timestamp
tradename
unreserves
url
webhooks
wildcard
wildcards
Mikko
Koivunalho
mikko
lib
Git
Mailmap
